import { Http, Loader } from '@zeedhi/core';
import { InstanceService } from '@/services/InstanceService';
import { DialogServiceCustom } from '@/services/DialogServiceCustom';
import { LoadingService } from '@zeedhi/common';
import { AlertServiceCustom } from '@/services/AlertServiceCustom';

export class DMTabIssueEditController {
	public demandManagementController = Loader.getInstance('DemandManagementController');

	public defaultTimeoutTimer: any;

	public isCallTypeS() {
		return this.demandManagementController.callType === 'S';
	}

	public getFormEditIssue() {
		return InstanceService.getInstanceZdForm('formDemandManagementI');
	}

	/** Eventos de fields */

	public onChangeClientEditCallModal(params: any) {
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			this.resetValidationsEDIT();
			const productInstance = InstanceService.getInstanceZdSelect('productEditI');
			productInstance.value = '';
			if (params.value) {
				this.resetInputtedValuesByClientEditField();
				productInstance.value = '';
				productInstance.datasource.filter.CDCLIENTE = params.component.value;
				try {
					await productInstance.datasource.get();
					this.demandManagementController.clientEditINotSelected = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.demandManagementController.clientEditINotSelected = true;
					this.demandManagementController.productEditINotSelected = true;
					this.demandManagementController.screenNameEditINotSelected = true;
					this.demandManagementController.versionEditINotSelected = true;
				}
			} else {
				this.demandManagementController.clientEditINotSelected = true;
				this.demandManagementController.productEditINotSelected = true;
				this.demandManagementController.screenNameEditINotSelected = true;
				this.demandManagementController.versionEditINotSelected = true;
			}
		}, 350);
	}

	public onChangeProductEditCallModal(params: any) {
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			this.resetValidationsEDIT();
			this.resetInputtedValuesByProductEditField();
			if (params.value) {
				const customerInstance = InstanceService.getInstanceZdSelect('customerEditI');
				const categoryInstance = InstanceService.getInstanceZdSelect('categoryEditI');

				const screenNameInstance = InstanceService.getInstanceZdSelect('screenNameEditI');
				categoryInstance.datasource.filter.APPLICATION_TYPE = 'I';
				screenNameInstance.datasource.filter.CDCLIENTE = customerInstance.selectedData.CDCLIENTE;
				screenNameInstance.datasource.filter.PRODUCT_ID = params.component.value.PRODUCT_FILTER_ID;
				try {
					await categoryInstance.datasource.get();
					await screenNameInstance.datasource.get();
					this.demandManagementController.productEditINotSelected = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.demandManagementController.productEditINotSelected = true;
					this.demandManagementController.screenNameEditINotSelected = true;
					this.demandManagementController.versionEditINotSelected = true;
				}
			} else {
				this.demandManagementController.productEditINotSelected = true;
				this.demandManagementController.screenNameEditINotSelected = true;
				this.demandManagementController.versionEditINotSelected = true;
			}
		}, 350);
	}

	public async onFocusProductEditCall() {
		const productInstance = InstanceService.getInstanceZdSelect('productEditI');
		delete productInstance.datasource.filter.CALL_RELATIONSHIP;
		delete productInstance.datasource.filter.CDPRODCOMP;

		await productInstance.datasource.get();
	}

	public onChangeScreenNameEditCallModal(params: any) {
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			this.resetValidationsEDIT();
			const versionIInstance = InstanceService.getInstanceZdSelect('versionEditI');
			const devVersionEditIInstance = InstanceService.getInstanceZdSelect('devVersionEditI');
			const customerInstanceEditi = InstanceService.getInstanceZdSelect('customerEditI');
			if (params.component.value) {
				versionIInstance.datasource.filter.PRODUCT_ID = params.component.value.NREXTERNAL_PRODUCT_ID;
				devVersionEditIInstance.datasource.filter.NREXTERNAL_PRODUCT_ID = params.component.value.NREXTERNAL_PRODUCT_ID;
				devVersionEditIInstance.datasource.filter.CDCLIENTE = customerInstanceEditi.selectedData.CDCLIENTE;
				try {
					await versionIInstance.datasource.get();
					await devVersionEditIInstance.datasource.get();
					this.demandManagementController.screenNameEditINotSelected = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.demandManagementController.screenNameEditINotSelected = true;
				}
			} else {
				this.demandManagementController.screenNameEditINotSelected = true;
				versionIInstance.value = '';
				devVersionEditIInstance.value = '';
			}
		}, 350);
	}

	public async saveEditIssue() { // SAVE EDIT
		const callFormInstance = this.getFormEditIssue();
		if (callFormInstance.validate()) {
			const callGridInstance = InstanceService.getInstanceTekGrid('gridDemandManagement');
			const customerInstance = InstanceService.getInstanceZdSelect('customerEditI');
			LoadingService.show();
			try {
				await Http.post('/productCallSave', {
					CALL_RELATIONSHIP: this.demandManagementController.callRelationship,
					ID: this.demandManagementController.callID,
					REFERENCE: callFormInstance.value.referenceEditI,
					CALL_LABELS: callFormInstance.value.labelsEditI,
					DESCRIPTION: callFormInstance.value.descriptionEditI,
					REPORTED_BY: callFormInstance.value.reportedByEditI,
					CDCLIENTE: customerInstance.selectedData.CDCLIENTE,
					PLATAFORMA: 'MOBILE',
					PROJECT_ID: callFormInstance.value.productEditI.PROJECT_ID,
					SGSISTEMA: null,
					TYPE: 'I',
					PRODUCT_VERSION: '0',
					PRODUCT_RELEASE: '0',
					MENU_ITEM_ID: callFormInstance.value.screenNameEditI.ID,
					DEVOPS_ID: callFormInstance.value.screenNameEditI.NREXTERNAL_PRODUCT_ID,
					MENU_VERSION_ID: callFormInstance.value.versionEditI,
					VERSION_DEV_ID: callFormInstance.value.devVersionEditI,
					CLASSIFICATION: callFormInstance.value.categoryEditI,
					APPOINTMENT_DATE: callFormInstance.value.appointmentDateEditI,
				});
				AlertServiceCustom.defaultSaveSuccessAlert();
				await callGridInstance.datasource.get();
			} catch (e) {
				LoadingService.hide();
				DialogServiceCustom.showErrorBackend(e);
			}
			LoadingService.hide();
		} else {
			DialogServiceCustom.messageErrorValidate();
		}
	}

	public resetValidationsEDIT() {
		const callFormInstance = this.getFormEditIssue();
		callFormInstance.resetValidation();
	}

	public resetInputtedValuesByClientEditField() {
		const callFormInstance = this.getFormEditIssue();
		callFormInstance.value.productEditI = '';
		callFormInstance.value.screenNameEditI = '';
		callFormInstance.value.versionEditII = '';
		callFormInstance.value.categoryEditI = '';
		callFormInstance.value.devVersionEditI = '';
	}

	public resetInputtedValuesByProductEditField() {
		const callFormInstance = this.getFormEditIssue();
		callFormInstance.value.screenNameEditI = '';
		callFormInstance.value.versionEditII = '';
		callFormInstance.value.categoryEditI = '';
		callFormInstance.value.devVersionEditI = '';
	}

	public resetInputtedValuesEDIT() {
		const callFormInstance = this.getFormEditIssue();
		callFormInstance.value.screenNameEdit = '';
		callFormInstance.value.versionEditI = '';
		callFormInstance.value.versionEditS = '';
	}
}
